-- Aero by Alza

--[[ Credits for this addon go to Cargor and his awesome VisualFX ]]

if FreeUI.config.aero == false then return end

local InCombatLockdown = InCombatLockdown
local Aero = CreateFrame"Frame"
local aero, percent, value
local running = {}
local next = next

local duration = 0.25

local function Anim(frame)
	aero = frame.aero
	percent  = aero.total / duration
	value = aero.start + aero.diff * percent
	if(value<=0) then value = 0.01 end
	frame:SetAlpha(value)
	frame:SetScale(value)
end

local function OnFinish(frame)
	frame:SetScale(1)
	frame:SetAlpha(1)
	if(frame.onfinishhide) then
		frame.onfinishhide = nil
		HideUIPanel(frame)
		frame.hiding = nil
	end
end

local function OnUpdate(self, elapsed)
	for i, frame in next, running do
		aero = frame.aero
		aero.total = aero.total + elapsed
		if(aero.total >= duration) then
			aero.total = 0
			running[i] = nil
			OnFinish(frame)
		else
			Anim(frame)
		end
	end
end
Aero:SetScript("OnUpdate", OnUpdate)

local function OnShow(frame)
	if(frame.hiding or running[frame] or InCombatLockdown()) then return end
	tinsert(running, frame)
	frame.aero.start = 0.01
	frame.aero.diff = 0.99
end

local function OnHide(frame)
	if(frame.hiding or running[frame] or InCombatLockdown()) then return end
	tinsert(running, frame)
	frame.aero.start = 1
	frame.aero.diff = -1
	frame.onfinishhide = true
	frame.hiding = true
	frame:Show()
end

function Aero:RegisterFrames(...)
	for i = 1, select("#", ...) do
		local arg = select(i, ...)
		if(type(arg)=="string") then
			local frame = _G[arg]
			if(not frame) then return print("Aero: |cff98F5FF", select(i, ...), "|r not found.") end
			frame.aero = frame.aero or {}
			frame.aero.total = 0
			frame:HookScript("OnShow", OnShow)
			frame:HookScript("OnHide", OnHide)
		else
			arg()
		end
	end
end

local addons = {}
local function OnEvent(self, event, addon)
	if(addons[addon]) then
		self:RegisterFrames(unpack(addons[addon]))
		addons[addon] = nil
	end
end
Aero:RegisterEvent"ADDON_LOADED"
Aero:SetScript("OnEvent", OnEvent)

function Aero:RegisterAddon(addon, ...)
	if(IsAddOnLoaded(addon)) then
		self:RegisterFrames(...)
	else
		local _, _, _, enabled = GetAddOnInfo(addon)
		if(enabled) then
			addons[addon] = {}
			for i = 1, select("#", ...) do
				local arg = select(i, ...)
				tinsert(addons[addon], arg)
			end
		end
	end
end

_G.Aero = Aero

--[[
Aero:RegisterFrames(...)
Use it to register show/hide animation to frames
Arguments can be either frame names or hook functions, seperated by commas
]]

Aero:RegisterFrames(
	"GameMenuFrame",
	"AudioOptionsFrame",
	"VideoOptionsFrame",
	"MacOptionsFrame",
	"InterfaceOptionsFrame",
	"OpacityFrame",
	"ItemRefTooltip",
	"OpenMailFrame",
	"StackSplitFrame",
	"ColorPickerFrame",
	"GuildInfoFrame",
	"PVPTeamDetails",
	"ChatConfigFrame",
	"GearManagerDialog",
	"ReputationDetailFrame",
	"PVPParentFrame",
	"LootFrame",
	"HelpFrame",
	"MailFrame",
	"ArenaFrame",
	"TradeFrame",
	"GossipFrame",
	"TabardFrame",
	"FriendsFrame",
	"MerchantFrame",
	"QuestLogFrame",
	"QuestLogDetailFrame",
	"SpellBookFrame",
	"PetStableFrame",
	"BattlefieldFrame",
	"GuildRegistrarFrame",
	"ArenaRegistrarFrame",
	"WorldMapFrame",
	"LFDParentFrame",
	"CharacterFrame"
)

--[[
Aero:RegisterAddon(addon, ...)
Use it to register frames that are created after addon loaded or on demand
Arguments:
	addon - addon's name
	... - either frame names or hook functions, seperated by commas
]]

Aero:RegisterAddon("Blizzard_CraftUI", "CraftFrame")
Aero:RegisterAddon("Blizzard_MacroUI", "MacroFrame")
Aero:RegisterAddon("Blizzard_AuctionUI", "AuctionFrame", "AuctionDressUpFrame")
Aero:RegisterAddon("Blizzard_GuildBankUI", "GuildBankFrame")
Aero:RegisterAddon("Blizzard_TalentUI", "PlayerTalentFrame")
Aero:RegisterAddon("Blizzard_TradeSkillUI", "TradeSkillFrame")
Aero:RegisterAddon("Blizzard_TrainerUI", "ClassTrainerFrame")
Aero:RegisterAddon("Blizzard_GMSurveyUI", "GMSurveyFrame")
Aero:RegisterAddon("Blizzard_AchievementUI", "AchievementFrame")
Aero:RegisterAddon("Blizzard_BattlefieldMinimap", "BattlefieldMinimap")
Aero:RegisterAddon("Blizzard_ItemSocketingUI", "ItemSocketingFrame")