-- aRaidCooldowns by Alza, modified.

local spells = {
	[GetSpellInfo(48477)] = 600, -- rebirth
	[GetSpellInfo(47883)] = 1800, -- ss
	[GetSpellInfo(19752)] = 600, -- di
	[GetSpellInfo(32182)] = 300, -- heroism
}

local width, height = 126, 12
local c = RAID_CLASS_COLORS[select(2, UnitClass("player"))]

local holder = CreateFrame("Frame", "aRaidCooldowns", UIParent)
if FreeUI.config.minimap_bottomright == true then
	holder:SetPoint("BOTTOM", Minimap, "TOP", 0, 16)
else
	holder:SetPoint("TOP", Minimap, "BOTTOM", 0, -2)
end
holder:SetHeight(height)
holder:SetWidth(width)

local floor, mod, format, ipairs = floor, mod, format, ipairs
local GetTime = GetTime
local bars = {}

local function SecondsToTime(seconds)
	local time = ""
	if(seconds >= 60) then
		time = format("%.0f.", floor(seconds * 0.016666666666667))
		seconds = mod(seconds, 60)
	end
	seconds = floor(seconds)
	if(seconds<10) then seconds = "0"..seconds end
	time = format("%s%s", time, seconds)
	return time
end

local function UpdateBars()
    local firsthidden = false
    local lastvisible
    for i, v in ipairs(bars) do
        if(i==1 and not v:IsShown()) then firsthidden = true end
        if(v:IsShown()) then
            v:ClearAllPoints()
	    if FreeUI.config.minimap_bottomright == true then
            	if(i==1 or firsthidden) then
                	v:SetPoint("TOP", holder, "BOTTOM")
            	else
                	v:SetPoint("BOTTOM", bars[lastvisible], "TOP", 0, 8)
            	end
	    else
            	if(i==1 or firsthidden) then
                	v:SetPoint("TOP", holder, "BOTTOM")
            	else
                	v:SetPoint("TOP", bars[lastvisible], "BOTTOM", 0, -8)
            	end
	    end
            firsthidden = false
            lastvisible = i
        end
    end
end

local OnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
	GameTooltip:AddLine(self.name, c.r, c.g, c.b)
	GameTooltip:AddDoubleLine(self.spellname, self.dur:GetText(), 1, 1, 1, 1, 1, 1)
	GameTooltip:Show()
end

local OnLeave = function()
	GameTooltip:Hide()
end

local function OnUpdate(self, elapsed)
	if(not self.start) then return end
	local time = GetTime()
	local progress = floor((time-self.start) / self.duration * 100)
	if(progress>=100) then
		self.start = nil
		self.duration = nil
		self:Hide()
		UpdateBars()
	else
		self:SetValue(progress)
		self.dur:SetFormattedText(SecondsToTime((self.start+self.duration)-time))
		self:SetStatusBarColor(1 - progress/100, progress/100, 0)
	end
end

local function OnMouseDown(self, button)
	if(button=="LeftButton") then
		SendChatMessage(format("CD: %s - %s - %s", self.name, self.spellname, self.dur:GetText()), "RAID")
	elseif(button=="RightButton") then
		self.start = nil
		self.duration = nil
		self:Hide()
		UpdateBars()
	end
end

local function CreateBar(index)
	local bar = CreateFrame("StatusBar", nil, UIParent)
	bar:SetStatusBarTexture(FreeUI.texture)
	-- bar:SetStatusBarColor(c.r, c.g, c.b)
	bar:SetMinMaxValues(0, 100)
	bar:SetHeight(height)
	bar:SetWidth(width)

	bar.dur = FreeUI.CreateFS(bar, 8, "LEFT")
	bar.dur:SetPoint("LEFT", bar, "TOPLEFT", 1, 1)

	bar.text = FreeUI.CreateFS(bar, 8, "RIGHT")
	bar.text:SetPoint("RIGHT", bar, "TOPRIGHT", -1, 1)

	bar:EnableMouse(true)
	bar:SetScript("OnEnter", OnEnter)
	bar:SetScript("OnLeave", OnLeave)
	bar:SetScript("OnUpdate", OnUpdate)
	bar:SetScript("OnMouseDown", OnMouseDown)

	local bg = CreateFrame("Frame", nil, bar)
	bg:SetBackdrop({
		bgFile = FreeUI.backdrop, 
		edgeFile = FreeUI.backdrop, 
		edgeSize = 1,
	})
	bg:SetBackdropColor(0, 0, 0, .5)
	bg:SetBackdropBorderColor(0, 0, 0)
	bg:SetPoint("TOPLEFT", bar, -1, 1)
	bg:SetPoint("BOTTOMRIGHT", bar, 1, -1)
	bg:SetFrameStrata("BACKGROUND")

	bars[index] = bar
	bar:Hide()
end

for i = 1, 10 do CreateBar(i) end

local function StartBar(unit, spellname, duration)
	local index = 1
	for i, v in ipairs(bars) do
		if(not v.start) then
			index = i
			break
		end
	end
	local bar = bars[index]
	bar.spellname = spellname
	bar.name = UnitName(unit)
	if spellname == GetSpellInfo(47883) then -- soulstone resurre...lol
		bar.text:SetText("Soulstone")
	elseif spellname == GetSpellInfo(19752) then -- divine inte... lol
		bar.text:SetText("DI")
	else
		bar.text:SetText(spellname)
	end
	bar.duration = duration
	bar.start = GetTime()
	bar:Show()
	UpdateBars()
end

holder:RegisterEvent"UNIT_SPELLCAST_SUCCEEDED"
holder:SetScript("OnEvent", function(self, event, unit, spell)
	if(unit:match("raid") and spells[spell]) then
		StartBar(unit, spell, spells[spell])
	end
end)

--[[ testmode ]]
local testui = TestUI or function() end
TestUI = function()
	testui()
	StartBar("player", "Пыщпыщьабилка", 180)
	StartBar("player", "Пыщпыщьабилка", 165)
	StartBar("player", "Пыщпыщьабилка", 130)
	StartBar("player", "Пыщпыщьабилка", 65)
end
SlashCmdList.TestUI = TestUI
SLASH_TestUI1 = "/testui"