--[[-------------------------------------------------------------------------
  Base code from oUF_Lily, Edited by Haleth:
    Please ask for permission before redistributing this work.
---------------------------------------------------------------------------]]

local parent, ns = ...
local oUF = ns.oUF

local colors = setmetatable({
	power = setmetatable({
		["MANA"] = {.9, .9, .9},
		["RAGE"] = {.9, .1, .1},
		["FUEL"] = {0, 0.55, 0.5},
		["FOCUS"] = {.9, .5, .1},
		["ENERGY"] = {.9, .9, .1},
		["AMMOSLOT"] = {0.8, 0.6, 0},
		["RUNIC_POWER"] = {.1, .9, .9},
		["POWER_TYPE_STEAM"] = {0.55, 0.57, 0.61},
		["POWER_TYPE_PYRITE"] = {0.60, 0.09, 0.17},
	}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})

--[[ Dropdown menu ]]
local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if(cunit == 'Vehicle') then
		cunit = 'Pet'
	end

	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

--[[ Short values ]]

local siValue = function(val)
	if(val >= 1e6) then
		return format("%.2fm", val * 0.000001)
	elseif(val >= 1e4) then
		return format("%.1fk", val * 0.001) 
	else
		return val
	end
end

--[[ Tags ]]

oUF.Tags['free:health'] = function(unit)
	if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	local min, max = UnitHealth(unit), UnitHealthMax(unit)
	if(unit=="target") then
		return format("|cffffffff%s|r %.0f", siValue(min), (min/max)*100)
	else
		return siValue(min)
	end
end
oUF.TagEvents['free:health'] = oUF.TagEvents.missinghp

oUF.Tags['free:maxhealth'] = function(unit)
	if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	local max = UnitHealthMax(unit)
	return max
end
oUF.TagEvents['free:maxhealth'] = oUF.TagEvents.missinghp

if FreeUI.config.healmode == true then
	oUF.Tags['free:missinghealth'] = function(unit)
		local min, max, name = UnitHealth(unit), UnitHealthMax(unit), UnitName(unit)
		if name and name:len() > 4 then name = name:sub(1, 4) end

		if not UnitIsConnected(unit) then
			return "Off" 
		elseif UnitIsDead(unit) then
			return "Dead"
		elseif UnitIsGhost(unit) then 
			return "Ghost"
		elseif min ~= max then
			return siValue(max-min)
		else
			return name
		end
	end
	oUF.TagEvents['free:missinghealth'] = oUF.TagEvents.missinghp
end

oUF.Tags['free:power'] = function(unit)
	local min, max = UnitPower(unit), UnitPowerMax(unit)
	if(min == 0 or max == 0 or not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then return end

	return siValue(min)
end
oUF.TagEvents['free:power'] = oUF.TagEvents.missingpp

--[[ Update health bar colour ]]

local UpdateHealth = function(self, event, unit)
	if(self.unit == unit) then
		local r, g, b, t
		if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit)) then
			r, g, b = .6, .6, .6
		elseif(unit == 'pet') then
			t = self.colors.happiness[GetPetHappiness()] or reactioncolors[UnitReaction(unit, "player")]
		elseif(UnitIsPlayer(unit)) then
			local _, class = UnitClass(unit)
			t = self.colors.class[class]
		else
			t = reactioncolors[UnitReaction(unit, "player")]
		end

		if(t) then
			r, g, b = t[1], t[2], t[3]
		end

		if(r) then
			if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then
				self.Health:SetStatusBarColor(0, 0, 0, 0)
				self.Power:SetStatusBarColor(r, g, b)
			else
				self.Health:SetStatusBarColor(r, g, b)
			end
		end

		if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then
			local min, max = UnitHealth(unit), UnitHealthMax(unit)
			self.Healthdef:SetPoint("LEFT", self.Health, self.Health:GetWidth() * (min/max), 0)
			if(not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit)) then
				self.Healthdef:Hide()
				if self.Text then self.Text:SetTextColor(.3, .7, 1) end
			else
				self.Healthdef:Show()
				self.Healthdef:SetVertexColor(self.ColorGradient(min / max, unpack(self.colors.smooth)))
				if self.Text then self.Text:SetTextColor(1, 1, 1) end
			end
		end
	end
end

--[[ Update health ]]

local PostUpdateHealth = function(Health, unit, min, max)
	if(UnitIsDead(unit)) then
		Health:SetValue(0)
	elseif(UnitIsGhost(unit)) then
		Health:SetValue(0)
	end

	if(Health.value) then
		if(unit=="target") then
			Health.value:SetTextColor(unpack(reactioncolors[UnitReaction("player", unit) or 5]))
		end
	end

	return UpdateHealth(Health:GetParent(), 'PostUpdateHealth', unit)
end

_G["BuffFrame"]:Hide()

--[[ Aura icon filter ]]

local PostUpdateIcon
do
	local playerUnits = {
		player = true,
		pet = true,
		vehicle = true,
	}

	PostUpdateIcon = function(icons, unit, icon, index, offset, filter, isDebuff)
		local texture = icon.icon
		local name, _, _, _, dtype, duration, expirationTime, unitCaster = UnitAura(unit, index, icon.filter)
		if(unit~="target") then icon:Show() end
		if(UnitIsFriend("player", unit) or playerUnits[icon.owner] or FreeUI.debuffFilter[name] or not icon.debuff or UnitIsPlayer(unit)) then
			icon:Show()
		else
			icon:Hide()
		end
	end
end

--[[ Update power value ]]

local PostUpdatePower = function(Power, unit, min, max)
	local Health = Power:GetParent().Health
	if(min == 0 or max == 0 or not UnitIsConnected(unit)) then
		Power:SetValue(0)
	elseif(UnitIsDead(unit) or UnitIsGhost(unit)) then
		Power:SetValue(0)
	end
end

local RAID_TARGET_UPDATE = function(self, event)
	local index = GetRaidTargetIndex(self.unit)
	if(index) then
		self.RIcon:SetText(ICON_LIST[index].."22|t")
	else
		self.RIcon:SetText()
	end
end

--[[ Temp buffs ]]

for i = 1, 2 do
	_G["TempEnchant"..i.."Border"]:Hide()
	bu = _G["TempEnchant"..i]
	ic = _G["TempEnchant"..i.."Icon"]
	du = _G["TempEnchant"..i.."Duration"]

	bu:ClearAllPoints()
	bu:SetHeight(36)
	bu:SetWidth(36)

	if i==1 then
		bu:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -6, 50)
	else
		bu:SetPoint("BOTTOMLEFT", bu[i-1], "TOPLEFT", 0, 2)
	end

	bu:SetBackdrop({
		edgeFile = FreeUI.backdrop, 
		edgeSize = 1, 
	})
	bu:SetBackdropBorderColor(0, 0, 0)

	ic:SetTexCoord(.08, .92, .08, .92)
	du:ClearAllPoints()
	du:SetPoint("BOTTOM", bu, 0, -2)
	du:SetFont(FreeUI.font, 8, "OUTLINEMONOCHROME")
	du:SetDrawLayer("OVERLAY")
	du:SetTextColor(1, 1, 1)
	du.SetTextColor = FreeUI.dummy
	du:SetShadowColor(0, 0, 0, 0)
end

--[[ Big function of doom ]]

local Shared = function(self, unit)
	self.menu = menu

	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)

	self:RegisterForClicks"anyup"
	self:SetAttribute("*type2", "menu")

	self.bg = CreateFrame("Frame", nil, self)
	self.bg:SetBackdrop({
		bgFile = FreeUI.backdrop,
		edgeFile = FreeUI.backdrop,
		edgeSize = 1,
	})
	self.bg:SetBackdropColor(0, 0, 0, .4)
	self.bg:SetBackdropBorderColor(0, 0, 0)
	self.bg:SetPoint("TOPLEFT", self, -1, 1)
	self.bg:SetPoint("BOTTOMRIGHT", self, 1, -1)
	self.bg:SetFrameStrata("BACKGROUND")

	--[[ Health ]]

	local Health = CreateFrame("StatusBar", nil, self)
	Health:SetStatusBarTexture(FreeUI.texture)
	Health:SetFrameStrata("LOW")

	Health.frequentUpdates = true

	Health:SetPoint("TOP")
	Health:SetPoint("LEFT")
	Health:SetPoint("RIGHT")

	self.Health = Health

	--[[ Health deficit colour (thanks nin) ]]

	if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then
		local Healthdef = Health:CreateTexture(nil, "BORDER")
		Healthdef:SetTexture(FreeUI.texture)
		Healthdef:SetPoint("TOPRIGHT", Health)
		Healthdef:SetPoint("BOTTOMRIGHT", Health)
		Healthdef:SetPoint("LEFT", Health, 0, 0)

		self.Healthdef = Healthdef
	end

	--[[ Power ]]

	local Power = CreateFrame("StatusBar", nil, self)
	Power:SetHeight(1)
	Power:SetStatusBarTexture(FreeUI.texture)

	if FreeUI.config.healmode == true and FreeUI.config.healmode_oldstyle == false then Power.colorPower = false else Power.colorPower = true end
	Power.frequentUpdates = true

	Power:SetPoint("LEFT")
	Power:SetPoint("RIGHT")
	Power:SetPoint("TOP", Health, "BOTTOM", 0, -1)

	self.Power = Power

	local Powerbg = CreateFrame("Frame", nil, self)
	Powerbg:SetBackdrop({
		bgFile = "",
		edgeFile = FreeUI.backdrop,
		edgeSize = 1,
	})
	Powerbg:SetBackdropColor(0, 0, 0, 0)
	Powerbg:SetBackdropBorderColor(0, 0, 0)
	Powerbg:SetPoint("TOPLEFT", self.Power, -1, 1)
	Powerbg:SetPoint("BOTTOMRIGHT", self.Power, 1, -1)
	Powerbg:SetFrameStrata("BACKGROUND")

	-- [[ Mouseover glow! ]]

	local glow = CreateFrame("Frame", nil, self)
	glow:SetBackdrop({
		edgeFile = FreeUI.glow,
		edgeSize = 4,
	})
	glow:SetPoint("TOPLEFT", self, -5, 5)
	glow:SetPoint("BOTTOMRIGHT", self, 5, -5)
	glow:SetBackdropBorderColor(228/255, 225/255, 16/255)
	glow:SetAlpha(0)

	self:HookScript("OnEnter", function(self)
		glow:SetAlpha(1)
		FreeUI.CreatePulse(glow)
	end)

	self:HookScript("OnLeave", function(self)
		glow:SetScript("OnUpdate", nil)
		glow:SetAlpha(0)
	end)

	--[[ Castbar ]]

	local Castbar = CreateFrame("StatusBar", nil, self)
	Castbar:SetStatusBarTexture("")
	Castbar:SetToplevel(true)

	local Spark = Castbar:CreateTexture(nil, "OVERLAY")
	Spark:SetTexture"Interface\\CastingBar\\UI-CastingBar-Spark"
	Spark:SetBlendMode"ADD"
	Spark:SetWidth(16)
	Castbar.Spark = Spark

	self.Castbar = Castbar

	local PostCastStart = function(Castbar, unit, spell, spellrank)
		if Castbar.Text then Castbar.Text:SetText(spell) end

		if self.Iconbg then
			if Castbar.interrupt then
				self.Iconbg:SetBackdropBorderColor(1, 0, 0)
			else
				self.Iconbg:SetBackdropBorderColor(0, 0, 0)
			end
		end
	end

	local PostCastStop = function(Castbar, unit)
		if Castbar.Text then Castbar.Text:SetText("") end
	end

	local PostCastStopUpdate = function(self, event, unit)
		if(unit ~= self.unit) then return end
		return PostCastStop(self.Castbar, unit)
	end

	--[[ Set up the layout ]]

	self.colors = colors

	self.disallowVehicleSwap = true

	self:RegisterEvent('UNIT_NAME_UPDATE', PostCastStopUpdate)
	table.insert(self.__elements, PostCastStopUpdate)

	Castbar.PostChannelStart = PostCastStart
	Castbar.PostCastStart = PostCastStart

	Castbar.PostCastStop = PostCastStop
	Castbar.PostChannelStop = PostCastStop

	Health.PostUpdate = PostUpdateHealth
	Power.PostUpdate = PostUpdatePower
end

local UnitSpecific = {
	pet = function(self)
		Shared(self)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(6)

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		self:RegisterEvent("UNIT_HAPPINESS", UpdateHealth)

		self:SetAttribute('initial-height', 8)
		self:SetAttribute('initial-width', 113)
	end,

	player = function(self)
		Shared(self)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(12)

		local HealthPoints = FreeUI.CreateFS(Health, 8)
		local MaxHealthPoints = FreeUI.CreateFS(Health, 8, "RIGHT")

		HealthPoints:SetPoint("LEFT", self, "TOPLEFT", 0, 6)
		HealthPoints:SetJustifyH("LEFT")
		MaxHealthPoints:SetPoint("RIGHT", self, "TOPRIGHT", 0, 6)

		self:Tag(HealthPoints, '[dead][offline][free:health]')
		self:Tag(MaxHealthPoints, '[free:maxhealth]')
		Health.value = HealthPoints

		local PowerPoints = FreeUI.CreateFS(Power, 8)
		local _, UnitPowerType = UnitPowerType("player")
		if(UnitPowerType=="MANA") then
			PowerPoints:SetPoint("LEFT", self, "TOPLEFT", 34, 6)
			PowerPoints:SetTextColor(.4, .7, 1)
		end

		self:Tag(PowerPoints, '[free:power]')

		Power.value = PowerPoints

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		Castbar.Text = FreeUI.CreateFS(Health, 8)
		Castbar.Text:SetAllPoints(Health)

		local IconFrame = CreateFrame("Frame", nil, Castbar)
		IconFrame:SetPoint("LEFT", self, "RIGHT", 3, 0)
		IconFrame:SetHeight(44)
		IconFrame:SetWidth(44)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
			self.Iconbg:SetBackdrop({
			bgFile = "",
			edgeFile = FreeUI.backdrop, 
			edgeSize = 1, 
			insets = {left = 0, right = 0, top = 0, bottom = 0}
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameStrata("LOW")

		local Buffs = CreateFrame("Frame", nil, self)
		Buffs.initialAnchor = "TOPRIGHT"

		if FreeUI.config.minimap_bottomright == true then
			Buffs:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -50, -50)
		else
			Buffs:SetPoint("TOPRIGHT", Minimap, "TOPLEFT", -10, 0)
		end

		local RaidIcon = FreeUI.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		Buffs["growth-y"] = "DOWN"
		Buffs["growth-x"] = "LEFT"
		Buffs["spacing-x"] = 3
		Buffs["spacing-y"] = 3

		Buffs:SetHeight(500)
		Buffs:SetWidth(310)
		Buffs.size = 32

		self.Buffs = Buffs

		local Debuffs = CreateFrame("Frame", nil, self)
		Debuffs.initialAnchor = "TOPRIGHT"
		Debuffs:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -6)
		Debuffs["growth-x"] = "LEFT"
		Debuffs["growth-y"] = "DOWN"
		Debuffs['spacing-x'] = 3
		Debuffs['spacing-y'] = 3

		Debuffs:SetHeight(60)
		Debuffs:SetWidth(229)
		Debuffs.num = 8
		Debuffs.size = 26

		self.Debuffs = Debuffs

		Debuffs.PostUpdateIcon = PostUpdateIcon

		Buffs.PostUpdateIcon = PostUpdateIcon

		self:SetAttribute('initial-height', 14)
		self:SetAttribute('initial-width', 229)
	end,

	target = function(self)
		Shared(self)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(12)

		local HealthPoints = FreeUI.CreateFS(Health, 8)

		HealthPoints:SetPoint("LEFT", self, "TOPLEFT", 0, 6)
		HealthPoints:SetJustifyH("LEFT")

		self:Tag(HealthPoints, '[dead][offline][free:health]')
		Health.value = HealthPoints

		local PowerPoints = FreeUI.CreateFS(Power, 8)
		PowerPoints:SetPoint("BOTTOMLEFT", Health.value, "BOTTOMRIGHT")

		self:Tag(PowerPoints, '[free:power]')

		Power.value = PowerPoints

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(self.Health:GetHeight())

		Castbar.Text = FreeUI.CreateFS(Health, 8)
		Castbar.Text:SetAllPoints(Health)

		local IconFrame = CreateFrame("Frame", nil, Castbar)
		IconFrame:SetPoint("LEFT", self, "RIGHT", 3, 0)
		IconFrame:SetHeight(44)
		IconFrame:SetWidth(44)

		local Icon = IconFrame:CreateTexture(nil, "BACKGROUND")
		Icon:SetAllPoints(IconFrame)
		Icon:SetTexCoord(.08, .92, .08, .92)

		Castbar.Icon = Icon

		self.Iconbg = CreateFrame("Frame", nil, IconFrame)
		self.Iconbg:SetBackdrop({
			bgFile = "",
			edgeFile = FreeUI.backdrop, 
			edgeSize = 1, 
			insets = {left = 0, right = 0, top = 0, bottom = 0}
		})
		self.Iconbg:SetPoint("TOPLEFT", IconFrame, "TOPLEFT", -1 , 1)
		self.Iconbg:SetPoint("BOTTOMRIGHT", IconFrame, "BOTTOMRIGHT", 1, -1)
		self.Iconbg:SetFrameStrata("LOW")

		local Name = FreeUI.CreateFS(Health, 8)
		Name:SetPoint("BOTTOMLEFT", Power.value, "BOTTOMRIGHT")
		Name:SetPoint("RIGHT", self)
		Name:SetJustifyH"RIGHT"
		Name:SetTextColor(1, 1, 1)

		self:Tag(Name, '[name]')
		self.Name = Name

		local RaidIcon = FreeUI.CreateFS(Health, 12)
		RaidIcon:SetPoint("RIGHT", self, "LEFT", -3, 0)

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		local Auras = CreateFrame("Frame", nil, self)
		Auras:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -6)
		Auras.initialAnchor = "TOPLEFT"
		Auras["growth-x"] = "RIGHT"
		Auras["growth-y"] = "DOWN"
		Auras['spacing-x'] = 3
		Auras['spacing-y'] = 3
		Auras.numDebuffs = 16
		Auras.numBuffs = 16
		Auras:SetHeight(500)
		Auras:SetWidth(229)
		Auras.size = 26
		Auras.gap = true

		self.Auras = Auras

		Auras.PostUpdateIcon = PostUpdateIcon

		self:SetAttribute('initial-height', 14)
		self:SetAttribute('initial-width', 229)
	end,

	focus = function(self)
		Shared(self)

		local Health = self.Health
		local Power = self.Power
		local Castbar = self.Castbar
		local Spark = Castbar.Spark

		Health:SetHeight(6)

		Castbar:SetAllPoints(Health)
		Castbar.Width = self:GetWidth()

		Spark:SetHeight(Health:GetHeight())

		local Debuffs = CreateFrame("Frame", nil, self)
		Debuffs:SetPoint("CENTER", UIParent, "CENTER", -216, -56)
		Debuffs.initialAnchor = "BOTTOMLEFT"
		Debuffs["growth-x"] = "RIGHT"
		Debuffs["growth-y"] = "DOWN"
		Debuffs["spacing-x"] = 3
		Debuffs:SetHeight(22)
		Debuffs:SetWidth(115)
		Debuffs.size = 22
		Debuffs.num = 5
		self.Debuffs = Debuffs

		Debuffs.PostUpdateIcon = PostUpdateIcon

		self:SetAttribute('initial-height', 8)
		self:SetAttribute('initial-width', 113)
	end,
}

do
	UnitSpecific.boss = function(self)
		if unit and unit:find("boss%d") then
			Shared(self)

			self:SetAttribute("type2", "focus")

			local Health, Power = self.Health, self.Power

			self:SetAttribute('initial-height', 12)
			self:SetAttribute('initial-width', 126)

			Health:SetHeight(10)

			local RaidIcon = FreeUI.CreateFS(Health, 8)
			RaidIcon:SetPoint("CENTER")

			self.RIcon = RaidIcon
			self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
			table.insert(self.__elements, RAID_TARGET_UPDATE)
		end
	end
end

do
	local range = {
		insideAlpha = 1,
		outsideAlpha = .5,
	}

	UnitSpecific.party = function(self)
		Shared(self)

		self.disallowVehicleSwap = false

		local Health, Power = self.Health, self.Power
		if FreeUI.config.healmode == true then
			self:SetAttribute('initial-height', 42)
			self:SetAttribute('initial-width', 42)
			Health:SetHeight(40)
		else
			self:SetAttribute('initial-height', 22)
			self:SetAttribute('initial-width', 38)
			Health:SetHeight(20)
		end

		if FreeUI.config.healmode == true then
			local Text = FreeUI.CreateFS(Health, 8, "CENTER")
			if FreeUI.config.healmode_oldstyle == true then Text:SetPoint("CENTER", 0, 0) else Text:SetPoint("CENTER", 1, 0) end
			self.Text = Text

			self:Tag(Text, '[free:missinghealth]')
		else
			local NotHere = FreeUI.CreateFS(Health, 8, "CENTER")
			NotHere:SetPoint("CENTER", 1, 0)

			self:Tag(NotHere, '[dead][offline]')
		end

		local RaidIcon = FreeUI.CreateFS(Health, 8)
		RaidIcon:SetPoint("CENTER")

		self.RIcon = RaidIcon
		self:RegisterEvent("RAID_TARGET_UPDATE", RAID_TARGET_UPDATE)
		table.insert(self.__elements, RAID_TARGET_UPDATE)

		local Leader = FreeUI.CreateFS(self, 8, "LEFT")
		Leader:SetText("l")
		Leader:SetPoint("TOPLEFT", Health, 1, -1)

		self.Leader = Leader

		local MasterLooter = FreeUI.CreateFS(self, 8, "RIGHT")
		MasterLooter:SetText("m")
		MasterLooter:SetPoint("TOPRIGHT", Health, 1, 0)

		self.MasterLooter = MasterLooter

		local rc = Health:CreateTexture(nil, "OVERLAY")
		rc:SetPoint"TOPLEFT"
		rc:SetHeight(16)
		rc:SetWidth(16)
		self.ReadyCheck = rc

		local dbh = Health:CreateTexture(nil, "OVERLAY")
		dbh:SetTexture(FreeUI.questborder)
		dbh:SetVertexColor(0, 0, 0, 1)
		dbh.DebuffHighlightAlpha = 1
		dbh:SetBlendMode("ADD")
		dbh:SetPoint("TOPLEFT", self, -3, 3)
		dbh:SetPoint("BOTTOMRIGHT", self, 3, -3)

		self.DebuffHighlight = dbh
		self.DebuffHighlightFilter = true

		local class = select(2, UnitClass("player"))
		if(class == "DRUID" or class == "PALADIN" or class == "PRIEST" or class == "SHAMAN") and FreeUI.config.healmode == true then
			local auras = {}
			local spellIDs = { 
				--Druid spells 
				48440, --Rejuvenation 
				48443, --Regrowth 
				48450, --Lifebloom 
				53249, --Wild Growth

				--Priest spells 
				6788,  --Weakened Soul 
				48113, --Prayer of Mending 
				48068, --Renew 
				48066, --Power Word: Shield

				--Shaman spells 
				61301, --Riptide  
				16237, --Ancestral Fortitude

				--Paladin spells 
				53563, --Beacon of Light 
				53601, --Sacred Shield

				-- Trial of the Crusader 
					-- Gormok the Impaler 
						66331,	-- Impale 
						66406,	-- Snobolded 
					-- Jormungar Behemoth 
						67618,	-- Paralytic Toxin 
						66869,  -- Burning Bile 
					-- Icehowl 
						66683,	-- Massive Crash 
					-- Lord Jaraxxus 
						66532,	-- Fel Fireball 
						66237,	-- Incinerate Flesh 
						66197,	-- Legion Flame 
					-- Faction Champions 
						65960,	-- Blind 
						65801,	-- Polymorph 
						65543,	-- Psychic Scream 
						66054,	-- Hex 
					-- The Twin Val'kyr 
						67283,	-- Dark Touch 
						67298,	-- Light Touch 
					-- Anub'arak 
						67574,	-- Pursued by Anub'arak 
						66013,	-- Penetrating Cold
						67847,	-- Expose Weakness 
				-- Icecrown Citadel 
					-- The Plagueworks  
						71127,	-- Mortal Wound 
					-- The Crimson Hall 
						70645,	-- Chains of Shadow 
						70435,	-- Rend Flesh 
					-- Lord Marrowgar 
						69065,	-- Impaled 
					-- Lady Deathwhisper 
						71289,	-- Dominate Mind 
						71204,	-- Touch of Insignificance 
					-- Gunship Battle 
						69651,	-- Wounding Strike 
					-- Deathbringer Saurfang 
						72293,	-- Mark of the Fallen Champion 
						72442,	-- Boiling Blood 
						72449,  -- Rune of Blood 
					-- Rotface 
						71224,	-- Mutated Infection 
					-- Festergut 
						69279,	-- Gas Spore 
						71218,	-- Vile Gas 
						72219,	-- Gastric Bloat 
					-- Professor Putricide 
						72549,	-- Malleable Goo 
						71278,	-- Choking Gas Bomb 
						72454,	-- Mutated Plague 
						72856,	-- Unbound Plague 
					-- Blood Princes 
						72796,	-- Glittering Sparks 
					-- Blood-Queen Lana'thel 
						71473,	-- Essence of the Blood Queen 
						71474,	-- Frenzied Bloodthirst 
						71265,	-- Swarming Shadows 
						70923,	-- Uncontrollable Frenzy 
					-- Valithria Dreamwalker 
						71283,	-- Gut Spray 
						71941,	-- Twisted Nightmares 
					-- Sindragosa 
						70126,	-- Frost Beacon 
						70157,	-- Ice Tomb 
						70127,	-- Mystic Buffet 
					-- The Lich King 
						70337,	-- Necrotic plague 
						69409,	-- Soul Reaper  
						72762,	-- Defile   
						68980,	-- Harvest Soul 
				--Ruby Sanctum 
					74502, -- Enervating Brand 
					75887, -- Blazing Aura   
					74562, -- Fiery Combustion 
					74567, -- Mark of Combustion (Fire) 
					74792, -- Soul Consumption 
					74795, -- Mark Of Consumption (Soul) 
				-- Other debuff 
					48468,	-- Insektenschwarm 
			}

			auras.onlyShowPresent = true
			auras.icons = {}
			for i, sid in pairs(spellIDs) do
				local icon = CreateFrame("Frame", nil, self)
				icon.spellID = sid
				if i < 13 then
					icon:SetWidth(12)
					icon:SetHeight(12)
					if i == 1 or i == 5 or i == 9 or i == 11 then
						icon:SetPoint("BOTTOMLEFT", Health, "BOTTOMLEFT", 1, 1)
					elseif i == 2 or i == 6 or i == 10 or i == 12 then
						icon:SetPoint("BOTTOMRIGHT", Health, "BOTTOMRIGHT", -1, 1)
					elseif i == 3 or i == 7 then
						icon:SetPoint("TOPLEFT", Health, "TOPLEFT", 1, -1)
					elseif i == 4 or i == 8 then
						icon:SetPoint("TOPRIGHT", Health, "TOPRIGHT", -1, -1)
					end
				else
					icon:SetPoint("CENTER", Health)
					icon:SetWidth(16)
					icon:SetHeight(16)
				end
				auras.icons[sid] = icon

				local cd = CreateFrame("Cooldown", nil, icon)
				cd:SetAllPoints(icon)
				cd:SetReverse()
				icon.cd = cd

				local count = FreeUI.CreateFS(icon, 8)
				count:SetPoint("TOP", icon, "TOP", 0, -2)
				icon.count = count

				FreeUI.CreateBD(icon)
			end
			self.AuraWatch = auras
		end

		local Threat = FreeUI.CreateFS(Health, 8)
		Threat:SetText("x")
		Threat:SetPoint("TOP", self, "TOP", 0, 0)
		self.Threat = Threat

		self.Range = range
	end
end

--[[ Register and activate style ]]

oUF:RegisterStyle("Free", Shared)
for unit,layout in next, UnitSpecific do
	oUF:RegisterStyle('Free - ' .. unit:gsub("^%l", string.upper), layout)
end

local spawnHelper = function(self, unit, ...)
	if(UnitSpecific[unit]) then
		self:SetActiveStyle('Free - ' .. unit:gsub("^%l", string.upper))
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	else
		self:SetActiveStyle'Free'
		local object = self:Spawn(unit)
		object:SetPoint(...)
		return object
	end
end

oUF:Factory(function(self)
	self:Spawn"focus":SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 566, 447)
	self:Spawn"pet":SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 450, 447)
	self:Spawn"player":SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 450, 420)
	self:Spawn"target":SetPoint("TOP", UIParent, "BOTTOM", 0, 300)
	spawnHelper(self, 'focus', "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 566, 447)
	spawnHelper(self, 'pet', "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 450, 447)
	spawnHelper(self, 'player', "BOTTOMLEFT", UIParent, "BOTTOMLEFT", 450, 420)
	spawnHelper(self, 'target', "TOP", UIParent, "BOTTOM", 0, 300)

	self:SetActiveStyle'Free - Party'

	local party = self:SpawnHeader(nil, nil, 'party,solo', 
		'showParty', true, 
		'showPlayer', true and FreeUI.config.healmode == true or false, 
		'showSolo', false, 
		'yoffset', -3, 
		'maxColumns', 5, 
		'unitsperColumn', 1, 
		'columnSpacing', 3, 
		'columnAnchorPoint', "RIGHT"
	)

	if FreeUI.config.healmode == false then
		party:SetPoint("BOTTOM", oUF_FreePlayerPlayer, "TOP", 0, 50)
	else
		party:SetPoint("CENTER", 280, -70)
	end

	local raid = oUF:SpawnHeader(nil, nil, 'raid,party,solo',
		'showPlayer', true,
		'showSolo', false,
		'showParty', false,
		'showRaid', true,
		'xoffset', 5,
		'yOffset', -4,
		'point', "TOP",
		'groupFilter', '1,2,3,4,5,6,7,8',
		'groupingOrder', '1,2,3,4,5,6,7,8',
		'groupBy', 'GROUP',
		'maxColumns', 7,
		'unitsPerColumn', 5,
		'columnSpacing', 5,
		'columnAnchorPoint', "RIGHT"
	)

	if FreeUI.config.healmode == false then
		if FreeUI.config.minimap_bottomright == true then
			raid:SetPoint("BOTTOMRIGHT", Minimap, "BOTTOMLEFT", -5, 0)
		else
			raid:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -50, 50)
		end
	else
		raid:SetPoint("CENTER", 280, -70)
	end

	for i = 1,MAX_BOSS_FRAMES do
		local t_boss = _G["Boss"..i.."TargetFrame"]
		t_boss:UnregisterAllEvents()
		t_boss.Show = FreeUI.dummy
		t_boss:Hide()
		_G["Boss"..i.."TargetFrame".."HealthBar"]:UnregisterAllEvents()
		_G["Boss"..i.."TargetFrame".."ManaBar"]:UnregisterAllEvents()
	end

	local boss = {}
	for i = 1, MAX_BOSS_FRAMES do
		boss[i] = self:Spawn("boss"..i, "oUF_FreeBoss"..i)
		boss[i]:SetPoint("BOTTOM", i == 1 and Minimap or boss[i-1], "TOP", 0, 2)
	end
end)

--[[ Targettarget text ]]

local tt = CreateFrame("Frame", nil, UIParent)
tt:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 5, 314)
tt:SetWidth(110)
tt:SetHeight(12)

ttt = FreeUI.CreateFS(tt, 8, "RIGHT")
ttt:SetAllPoints(tt)

tt:SetScript("OnUpdate", function()
	if(UnitName("targettarget")==UnitName("player")) then
		ttt:SetText("> YOU <")
		ttt:SetTextColor(1, 0, 0)
	else
		ttt:SetText(UnitName"targettarget")
		ttt:SetTextColor(1, 1, 1)
	end
end)